Imports Palisade.RdkPia

Public Class Form1

    Dim WithEvents myRDKApp As RDKApplication
    Dim numMonths As Int32
    Dim inflationMeanFirstMonth As Double
    Dim inflationStdDev As Double
    Dim inflationMinimum As Double

    Dim priceInputs() As RDKInput
    Dim quantityInputs() As RDKInput
    Dim inflationInputs() As RDKInput

    Private Const TWIPS_PER_PIXEL As Int32 = 15

    Private Sub ButtonSimulate_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles ButtonSimulate.Click

        myRDKApp = New RDKApplication
        myRDKApp.Initialize(False)

        numMonths = Me.TextBoxNumMonths.Text
        inflationMeanFirstMonth = Me.TextBoxNormalMean.Text / 100.0
        inflationStdDev = Me.TextBoxNormalStdDev.Text / 100.0
        inflationMinimum = Me.TextBoxNormalMin.Text / 100.0

        AddInputs()

        AddOutputs()

        ' define simulation settings
        With myRDKApp.SimSettings
            .NumIterations = 2000
        End With

        myRDKApp.Simulate()

        DisplayOutputStatistics()

        GraphTotalOutput()

        GenerateSummaryGraphOfRangeOfOutputs()

        myRDKApp.Free()
    End Sub

    Private Function GetLognormalDistribution(ByVal mean As Double, ByVal stdDev As Double) As RDKDistribution
        Dim distr As RDKDistribution
        distr = New RDKDistribution
        With distr
            .DistType = RDKDistributionType.RDKDistTypeLOGNORM
            .Arg(1) = mean
            .Arg(2) = stdDev
        End With
        GetLognormalDistribution = distr
    End Function

    Private Function GetTriangularDistribution(ByVal min As Double, ByVal ml As Double, ByVal max As Double) As RDKDistribution
        Dim distr As RDKDistribution
        distr = New RDKDistribution
        With distr
            .DistType = RDKDistributionType.RDKDistTypeTRIANG
            .Arg(1) = min
            .Arg(2) = ml
            .Arg(3) = max
        End With
        GetTriangularDistribution = distr
    End Function

    Private Sub AddInputs()

        ReDim priceInputs(numMonths - 1)
        ReDim quantityInputs(numMonths - 1)
        ReDim inflationInputs(numMonths - 1)

        Dim i As Long
        Dim distr As RDKDistribution
        For i = 1 To numMonths
            With myRDKApp.Inputs
                distr = GetLognormalDistribution(Me.TextBoxLognormalMean.Text, Me.TextBoxLognormalStdDev.Text)
                priceInputs(i - 1) = .Add("Price_Month" & i, distr)

                distr = GetTriangularDistribution(Me.TextBoxTriangularMin.Text, Me.TextBoxTriangularML.Text, Me.TextBoxTriangularMax.Text)
                quantityInputs(i - 1) = .Add("Quantity_Month" & i, distr)

                ' no default distribution
                inflationInputs(i - 1) = .Add("Inflation_Month" & i)
            End With
        Next i

    End Sub

    Private Sub AddOutputs()
        Dim rangeOutputNames(numMonths - 1) As String
        Dim i As Int32
        For i = 1 To numMonths
            rangeOutputNames(i - 1) = "Month" & i
        Next
        ' add range of outputs
        myRDKApp.Outputs.AddRange("Monthly Totals", rangeOutputNames)
        myRDKApp.Outputs.Add("Total")
    End Sub


    Private Sub myRDKApp_Iteration(ByRef whichSim As Integer, ByRef whichIter As Integer, ByRef cancel As Boolean) Handles myRDKApp.Iteration
        Dim i As Int32
        Dim theTotal As Double
        ' sample inflation inputs
        For i = 1 To numMonths
            If i = 1 Then
                inflationInputs(i - 1).RISKNormal(inflationMeanFirstMonth, inflationStdDev, inflationMinimum)
            Else
                inflationInputs(i - 1).RISKNormal(inflationInputs(i - 2).Value, inflationStdDev, inflationMinimum)
            End If
        Next i
        ' calculate output values
        For i = 1 To numMonths
            myRDKApp.Outputs(i).Value = priceInputs(i - 1).Value * quantityInputs(i - 1).Value * (1 + inflationInputs(i - 1).Value)
            theTotal = theTotal + myRDKApp.Outputs(i).Value
        Next i
        myRDKApp.Outputs("Total").Value = theTotal
    End Sub

    Private Sub DisplayOutputStatistics()
        With Me.myRDKApp.Outputs("Total").Results
            Me.TextBoxTotalMean.Text = .Statistics.Mean.ToString("0")
            Me.TextBoxTotalStdDev.Text = .Statistics.StdDeviation.ToString("0")
        End With
    End Sub

    Private Sub GraphTotalOutput()

        ' graph output
        ' first create file with graph
        Dim tempFilePath As String
        With myRDKApp.GraphDefaults
            .Destination = RDKGraphDestination.RDKJPGFile
            ' this line makes a temp file
            tempFilePath = IO.Path.GetTempFileName
            ' I don't want a file, just a path ending in .jpg
            Kill(tempFilePath)
            tempFilePath = tempFilePath & ".jpg"
            .DestinationFile = tempFilePath
            .PictureHeight = Me.PictureBoxOutputHistogram.Height * TWIPS_PER_PIXEL
            .PictureWidth = Me.PictureBoxOutputHistogram.Width * TWIPS_PER_PIXEL
            .DisplayMean = True
        End With

        myRDKApp.Outputs("Total").Results.Graph(RDKResultCurveType.RDKResultCurveTypeHistogram)
        Me.PictureBoxOutputHistogram.Image = New Bitmap(tempFilePath)

    End Sub

    Private Sub GenerateSummaryGraphOfRangeOfOutputs()

        ' graph output
        ' first create file with graph
        Dim tempFilePath As String
        With myRDKApp.GraphDefaults
            .Destination = RDKGraphDestination.RDKJPGFile
            ' this line makes a temp file
            tempFilePath = IO.Path.GetTempFileName
            ' I don't want a file, just a path ending in .jpg
            Kill(tempFilePath)
            tempFilePath = tempFilePath & ".jpg"
            .DestinationFile = tempFilePath
            .PictureHeight = Me.PictureBoxOutputHistogram.Height * TWIPS_PER_PIXEL
            .PictureWidth = Me.PictureBoxOutputHistogram.Width * TWIPS_PER_PIXEL
            .DisplayMean = True
        End With

        myRDKApp.Outputs(1).Results.GraphSummary()
        Me.PictureBoxSummaryGraph.Image = New Bitmap(tempFilePath)

    End Sub

End Class
